from django.shortcuts import render, redirect
from django.db.models import Q
from django.contrib import messages
from django.http import HttpResponse, JsonResponse
from django.views.generic import TemplateView
from django_eveonline_connector.models import EveCharacter, EveEntity
from django_eveonline_connector.models import EveAsset, EveJumpClone, EveContact, EveContract, EveSkill, EveJournalEntry, EveTransaction
from django_eveonline_connector.models import EveClient
from django_eveonline_connector.utilities.esi.universe import resolve_id
from django_datatables_view.base_datatable_view import BaseDatatableView
from django.contrib.auth.decorators import login_required, permission_required
from django.utils.html import escape
import json 

# Utilities 
@login_required
def get_entity_info(request):
    external_id = None 
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    else:
        external_id = request.GET['external_id']
    print(external_id)
    
    entity = resolve_id(external_id)

    if not entity:
        return HttpResponse(status=404)

    if entity['type'] == "character":
        response = EveClient.call('get_characters_character_id', character_id=external_id).data
        return JsonResponse({
            "type": "character",
            "name": response['name'],
            "external_id": external_id,
        })
    elif entity['type'] == "corporation":
        response = EveClient.call('get_corporations_corporation_id', corporation_id=external_id).data
        return JsonResponse({
            "type": "corporation",
            "name": response['name'],
            "ticker": response['ticker'],
            "external_id": external_id,
            "alliance_id": response['alliance_id'],
        })
    elif entity['type'] == "alliance":
        response = EveClient.call('get_alliances_alliance_id', alliance_id=external_id).data
        return JsonResponse({
            "type": "alliance", 
            "name": response['name'],
            "ticker": response['ticker'],
            "external_id": external_id,
        })
    else:
        return HttpResponse(status=500)



# Character Lookups
@login_required
@permission_required('django_eveonline_connector.view_eveasset', raise_exception=True)
def get_assets(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return AssetJson.as_view()(request)


@login_required
@permission_required('django_eveonline_connector.view_evejumpclone', raise_exception=True)
def get_clones(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return CloneJson.as_view()(request)


@login_required
@permission_required('django_eveonline_connector.view_evecontact', raise_exception=True)
def get_contacts(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return ContactJson.as_view()(request)


@login_required
@permission_required('django_eveonline_connector.view_evecontract', raise_exception=True)
def get_contracts(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return ContractJson.as_view()(request)


@login_required
@permission_required('django_eveonline_connector.view_evejournalentry', raise_exception=True)
def get_journal(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return JournalJson.as_view()(request)


@login_required
@permission_required('django_eveonline_connector.view_evetransaction', raise_exception=True)
def get_transactions(request):
    if 'external_id' not in request.GET:
        return HttpResponse(status=400)
    if not EveEntity.objects.filter(external_id=request.GET.get('external_id')):
        return HttpResponse(status=404)
    return TransactionJson.as_view()(request)


# JSON Class Views
class AssetJson(BaseDatatableView):
    model = EveAsset
    columns = ['item_name', 'location_name', 'quantity']
    order_columns = ['item_name', 'location_name', 'quantity']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[value]', None)
        if search:
            qs = qs.filter(Q(item_name__istartswith=search) | Q(location_name__istartswith=search))

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(entity__external_id=external_id, location_flag="Hangar")

    def prepare_results(self, qs):
        json_data = []
        for item in qs:
            json_data.append([
                item.item_name,
                item.location_name,
                item.quantity,
            ])
        return json_data


class CloneJson(BaseDatatableView):
    model = EveJumpClone
    columns = ['location', 'implants']
    order_columns = ['location', 'implants']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[value]', None)
        if search:
            qs = qs.filter(Q(location__istartswith=search)
                           | Q(implants__contains=search))

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(Q(entity__external_id=external_id))

    def prepare_results(self, qs):
        json_data = []
        for item in qs:
            json_data.append([
                item.location,
                item.implants.replace(",", "<br>"),
            ])
        return json_data


class ContactJson(BaseDatatableView):
    model = EveContact
    columns = ['name', 'contact_type', 'standing']
    order_columns = ['name', 'contact_type', 'standing']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[value]', None)
        if search:
            qs = qs.filter(
                Q(contact_name__istartswith=search) | 
                Q(contact_type__istartswith=search)
                )

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(Q(entity__external_id=external_id))

    def prepare_results(self, qs):
        json_data = []
        for item in qs:
            json_data.append([
                '<a onclick="setModalID(%s)" href="#" data-toggle="modal" data-target="#entityModal">%s</a>' % (item.contact_id, item.contact_name),
                item.contact_type.title(),
                item.standing
            ])
        return json_data


class ContractJson(BaseDatatableView):
    model = EveContract
    columns = ['date_issued', 'status',
               'type', 'issuer_name', 'assignee_name', 'items']
    order_columns = ['date_issued', 'status',
                     'type', 'issuer_name', 'assignee_name', 'items']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[value]', None)
        if search:
            qs = qs.filter(Q(issuer_name__istartswith=search) |
                           Q(assignee_name__istartswith=search) |
                           Q(type__istartswith=search) | 
                           Q(status__istartswith=search)
                           )

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(Q(entity__external_id=external_id))

    def prepare_results(self, qs):
        json_data = []
        for contract in qs:
            date_issued = contract.date_issued
            status = contract.status
            type = contract.type
            issuer_name = contract.issuer_name
            if contract.acceptor_name:
                assignee_name = contract.acceptor_name
            elif contract.issued_to:
                assignee_name = contract.assignee_name
            else:
                assignee_name = "Public"
            actions = """
                <button class="text-center btn btn-success" data-toggle="modal" data-target="#view_%s"><i class="fa fa-eye"></i></button>
                <div class="modal fade col-md-12" id="view_%s" data-backdrop="false">
                    <div class="modal-dialog">
                        <div class="modal-content">
                            <div class="modal-header">
                                <h4>View Contract</h4>
                            </div>
                            <div class="modal-body">
                               <p>%s</p>
                            </div>
                            <div class="modal-footer">
                                <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
                            </div>
                        </div>
                    </div>
                </div>
            """ % (contract.pk, contract.pk, contract.items.replace("\n", "<br>"))

            json_data.append([
                date_issued.strftime("%m-%d-%Y"),
                status,
                type.replace("_", " ").upper(),
                issuer_name,
                assignee_name,
                actions
            ])
        return json_data


class JournalJson(BaseDatatableView):
    model = EveJournalEntry
    columns = ['date', 'ref_type', 'first_party_name', 'second_party_name', 'amount']
    order_columns = ['date', 'ref_type', 'first_party_name', 'second_party_name', 'amount']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[value]', None)
        if search:
            qs = qs.filter(Q(ref_type__istartswith=search) |
                           Q(first_party_name__istartswith=search) |
                           Q(second_party_name__istartswith=search))

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(Q(entity__external_id=external_id))

    def prepare_results(self, qs):
        json_data = []
        # resolve custom template responses
        for entry in qs:
            first_party_ext = "jpg"
            second_party_ext = "jpg"
            if entry.first_party_type == "corporation":
                first_party_ext = "png"
            if entry.second_party_type == "corporation":
                second_party_ext = "png"
            # add avatars for first party field
            if entry.first_party_type == "corporation" or entry.first_party_type == "character":
                entry_first_party = """
                <img width="32px" src="https://imageserver.eveonline.com/%s/%s_64.%s" class="img-circle img-bordered-sm" alt="Avatar">
                %s 
                """ % (entry.first_party_type.title(), entry.first_party_id, first_party_ext, entry.first_party_name)
            else:
                entry_first_party = entry.ref_type
            # add avatars for second party field
            if entry.second_party_type == "corporation" or entry.second_party_type == "character":
                entry_second_party = """
                <img width="32px" src="https://imageserver.eveonline.com/%s/%s_64.%s" class="img-circle img-bordered-sm" alt="Avatar">
                %s 
                """ % (entry.second_party_type.title(), entry.second_party_id, second_party_ext, entry.second_party_name)
            else:
                entry.second_party_name = entry.ref_type
            # clean up amount html
            if entry.amount < 0:
                amount_color = "red"
            else:
                amount_color = "green"
            entry_amount = """
                <p><span style="color: %s">%s</span></p>
            """ % (amount_color, f'{entry.amount:,}')

            json_data.append([
                entry.date.strftime("%m-%d-%Y"),
                entry.ref_type.title(),
                entry_first_party,
                entry_second_party,
                entry_amount,
            ])

        return json_data


class TransactionJson(BaseDatatableView):
    model = EveTransaction
    columns = ['client_name', 'item_name', 'quantity', 'unit_price']
    order_columns = ['client_name', 'item_name', 'quantity', 'unit_price']

    def filter_queryset(self, qs):
        # implement searching
        search = self.request.GET.get('search[unit_price]', None)
        if search:
            qs = qs.filter(Q(item_name__istartswith=search) |
                           Q(client__istartswith=search)
                           )

        # return character
        external_id = self.request.GET.get('external_id')
        return qs.filter(Q(entity__external_id=external_id))

    def prepare_results(self, qs):
        json_data = []
        # resolve custom template responses
        for transaction in qs:
            client_ext = "jpg"
            if transaction.client_type == "corporation":
                client_ext = "png"
            # add avatars for client_name
            if transaction.client_type.lower() == "corporation" or transaction.client_type.lower() == "character":
                transaction_client = """
                <img width="32px" src="https://imageserver.eveonline.com/%s/%s_64.%s" class="img-circle img-bordered-sm" alt="Avatar">
                %s 
                """ % (transaction.client_type.title(), transaction.client_id, client_ext, transaction.client_name)
            else:
                transaction_client = transaction.client_type
            # clean up unit_price html
            if transaction.is_buy:
                amount_color = "red"
            else:
                amount_color = "green"
            transaction_amount = """
                <p><span style="color: %s">%s</span></p>
            """ % (amount_color, f'{transaction.unit_price:,}')

            json_data.append([
                transaction_client,
                transaction.item_name,
                transaction.quantity,
                transaction_amount,
            ])

        return json_data



# Generated by Django 2.2.10 on 2020-09-29 15:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_eveonline_connector', '0031_evecharacterinfo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eveasset',
            name='item_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='eveasset',
            name='location_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='evecontract',
            name='end_location_id',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='evecontract',
            name='start_location_id',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='evejournalentry',
            name='context_id',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='evejournalentry',
            name='external_id',
            field=models.BigIntegerField(unique=True),
        ),
        migrations.AlterField(
            model_name='evejumpclone',
            name='location_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='evetransaction',
            name='journal_ref_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='evetransaction',
            name='location_id',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='evetransaction',
            name='transaction_id',
            field=models.BigIntegerField(unique=True),
        ),
    ]



import os
from sys import path
from django.core import serializers
from django.db import migrations

fixture_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../fixtures'))
fixture_filename = 'initial_scopes.json'

def load_fixture(apps, schema_editor):
    fixture_file = os.path.join(fixture_dir, fixture_filename)

    fixture = open(fixture_file, 'rb')
    objects = serializers.deserialize('json', fixture, ignorenonexistent=True)
    for obj in objects:
        obj.save()
    fixture.close()

def unload_fixture(apps, schema_editor):
    "Brutally deleting all entries for this model..."

    MyModel = apps.get_model("django_eveonline_connector", "EveScope")
    MyModel.objects.all().delete()

class Migration(migrations.Migration):  

    dependencies = [
        ('django_eveonline_connector', '0001_initial'),
    ]

    operations = [
        # migrations.RunPython(load_fixture, reverse_code=unload_fixture),
    ]


# Generated by Django 2.2.13 on 2020-10-04 00:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_eveonline_connector', '0033_auto_20200930_1805'),
    ]

    operations = [
        migrations.CreateModel(
            name='EveStructure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corporation_id', models.BigIntegerField()),
                ('fuel_expires', models.DateTimeField(null=True)),
                ('next_reinforce_apply', models.DateTimeField(null=True)),
                ('next_reinforce_hour', models.BigIntegerField(null=True)),
                ('next_reinforce_weekday', models.BigIntegerField(null=True)),
                ('profile_id', models.BigIntegerField()),
                ('reinforce_hour', models.BigIntegerField()),
                ('reinforce_weekday', models.BigIntegerField(null=True)),
                ('services', models.TextField(null=True)),
                ('state', models.CharField(max_length=64)),
                ('state_timer_end', models.DateTimeField(null=True)),
                ('state_timer_start', models.DateTimeField(null=True)),
                ('structure_id', models.BigIntegerField()),
                ('system_id', models.BigIntegerField()),
                ('unanchors_at', models.DateTimeField(null=True)),
                ('owner_id', models.BigIntegerField()),
                ('solar_system_id', models.BigIntegerField()),
                ('type_id', models.BigIntegerField(null=True)),
                ('name', models.CharField(max_length=256)),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-08 05:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_eveonline_connector', '0022_auto_20200303_1841'),
    ]

    operations = [
        migrations.CreateModel(
            name='EveTransaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('client_id', models.IntegerField()),
                ('date', models.DateTimeField()),
                ('is_buy', models.BooleanField()),
                ('is_personal', models.BooleanField()),
                ('journal_ref_id', models.IntegerField()),
                ('location_id', models.IntegerField()),
                ('quantity', models.IntegerField()),
                ('transaction_id', models.IntegerField(unique=True)),
                ('type_id', models.IntegerField()),
                ('unit_price', models.FloatField()),
                ('client_name', models.CharField(max_length=64)),
                ('client_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=64)),
                ('location_name', models.CharField(max_length=64)),
                ('type_name', models.CharField(max_length=64)),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EveJournalEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('external_id', models.IntegerField(unique=True)),
                ('amount', models.FloatField(blank=True, null=True)),
                ('balance', models.FloatField(blank=True, null=True)),
                ('tax', models.FloatField(blank=True, null=True)),
                ('tax_receiver_id', models.IntegerField(blank=True, null=True)),
                ('context_id', models.IntegerField(blank=True, null=True)),
                ('context_id_type', models.CharField(blank=True, max_length=32, null=True)),
                ('date', models.DateTimeField()),
                ('description', models.TextField()),
                ('reason', models.TextField()),
                ('ref_type', models.CharField(max_length=128)),
                ('first_party_id', models.IntegerField(blank=True, null=True)),
                ('second_party_id', models.IntegerField(blank=True, null=True)),
                ('first_party_name', models.CharField(max_length=64)),
                ('first_party_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=32)),
                ('second_party_name', models.CharField(max_length=64)),
                ('second_party_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=32)),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EveContract',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contract_id', models.IntegerField(unique=True)),
                ('acceptor_id', models.IntegerField()),
                ('assignee_id', models.IntegerField()),
                ('buyout', models.FloatField(blank=True, null=True)),
                ('collateral', models.FloatField(blank=True, null=True)),
                ('price', models.FloatField(blank=True, null=True)),
                ('reward', models.FloatField(blank=True, null=True)),
                ('volume', models.FloatField(blank=True, null=True)),
                ('date_accepted', models.DateTimeField(blank=True, null=True)),
                ('date_completed', models.DateTimeField(blank=True, null=True)),
                ('date_expired', models.DateTimeField()),
                ('date_issued', models.DateTimeField()),
                ('days_to_complete', models.IntegerField(blank=True, null=True)),
                ('for_corporation', models.BooleanField()),
                ('availability', models.CharField(max_length=32)),
                ('issuer_corporation_id', models.IntegerField()),
                ('issuer_id', models.IntegerField()),
                ('start_location_id', models.IntegerField(blank=True, null=True)),
                ('end_location_id', models.IntegerField(blank=True, null=True)),
                ('status', models.CharField(max_length=32)),
                ('title', models.TextField(blank=True, null=True)),
                ('type', models.CharField(max_length=16)),
                ('acceptor_name', models.CharField(max_length=64)),
                ('assignee_name', models.CharField(max_length=64)),
                ('issuer_name', models.CharField(max_length=64)),
                ('issuer_corporation_name', models.CharField(max_length=64)),
                ('acceptor_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=32)),
                ('assignee_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=32)),
                ('issuer_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=64)),
                ('items', models.TextField()),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EveContact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact_id', models.IntegerField()),
                ('is_blocked', models.BooleanField(default=False)),
                ('is_watched', models.BooleanField(default=False)),
                ('label_ids', models.TextField()),
                ('standing', models.FloatField()),
                ('contact_name', models.CharField(max_length=64)),
                ('contact_type', models.CharField(choices=[('character', 'character'), ('corporation', 'corporation'), ('alliance', 'alliance')], max_length=32)),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EveSkill',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active_skill_level', models.IntegerField()),
                ('trained_skill_level', models.IntegerField()),
                ('skill_id', models.IntegerField()),
                ('skillpoints_in_skill', models.IntegerField()),
                ('skill_name', models.CharField(max_length=64)),
                ('skill_group', models.CharField(max_length=64)),
                ('entity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_eveonline_connector.EveEntity')),
            ],
            options={
                'unique_together': {('entity', 'skill_name')},
            },
        ),
    ]
